

/* ***********************************************
 * file		: fat.h
 * 		: head file for fat.c
 * 		: 
 * date		: 2008-07-03
 * **********************************************/
#ifndef FAT_H_
#define FAT_H_

#define FAT16	0x06
#define FAT32	0x0b

#define TRUE    0xff
#define FALSE   0x00

/* 主引导记录 512字节(磁盘第一扇区) */
struct master_boot_record {
    uint8_t mbr_boot_code[446];		/* 引导代码 */
    uint8_t mbr_partition_table[64];	/* 四个分区表 */
    uint8_t mbr_boot_sect_sig0;		/* 0x55 */
    uint8_t mbr_boot_sect_sig1;		/* 0xaa */
};

/* 分区表 16字节 */
struct partition_table{
    uint8_t  pt_active;			/* 分区活动标志 */
    uint8_t  pt_start_head;		/* 分区开始磁头 */
    uint16_t pt_start_sect_cyl; 	/* 开始扇区柱面 */
    uint8_t  pt_part_type;		/* 分区类型 */
    uint8_t  pt_end_head;
    uint16_t pt_end_sect_cyl;
    uint32_t pt_preceding_sector;	/* 分区之前扇区数 */
    uint32_t pt_length_of_partition;	/* 分区总扇区数 */
};

/* 分区启动扇区 512字节(分区第一扇区) */
struct boot_sector_710{
    uint8_t bs_jump_boot[3];	/* jump inst E9xxx or EBxx90 */
    uint8_t bs_oem_name[8];	/* OEM name and version */
    uint8_t bs_bpb[53];		/* BIOS parameter block */
    uint8_t bs_ext[26];		/* bootsector extension */
    uint8_t bs_boot_code[418];	/* pad so structure is 512b */
    uint8_t bs_boot_sig2;	/* 0x00(FAT32) */
    uint8_t bs_boot_sig3;	/* 0x00(FAT32) */
    uint8_t bs_boot_sig0;	/* 0x55 */
    uint8_t bs_boot_sig1; 	/* 0xaa */
};

/* BPB for DOS7.10 */
struct bpb_710{
    uint16_t bpb_byts_per_sect; /* 每扇区字节数 */
    uint8_t  bpb_sect_per_clus; /* 每簇扇区数 */
    uint16_t bpb_rsvd_sect_cnt; /* 保留扇区数 */
    uint8_t  bpb_num_FATs;	/* FAT表份数 */
    uint16_t bpb_root_ent_cnt;  /* FAT16根目录中目录项数 */
    uint16_t bpb_tot_sect_16;   /* 分区总扇区数(16bit) */
    uint8_t  bpb_media;		/* 磁盘介质类型 */
    uint16_t bpb_FATsz;		/* FAT表大小(单位扇区) */
    uint16_t bpb_sect_per_trk;	/* 每磁道扇区数 */
    uint16_t bpb_num_heads;	/* 磁头数 */
    uint32_t bpb_hidd_sect;	/* 隐藏扇区数 */
    uint32_t bpb_tot_sect_32;	/* 分区总扇区数(32bit) */
    /* 以下为FAT32专用 */
    uint32_t bpb_FAT32sz;	/* FAT表大小 */
    uint16_t bpb_ext_flags;
    uint16_t bpb_FS_ver;
    uint32_t bpb_root_cluster;  /* 根目录起始簇号 */
    uint16_t bpb_FS_info;
    uint16_t bpb_bk_boot_sect;
    uint8_t  bpb_rsvd[12];
};

/* Bootsector Extension 26B */
struct bs_ext{
    uint8_t  bs_drv_num;	/* 驱动器参数 */
    uint8_t  bs_rsvd;		/* for NT */
    uint8_t  bs_boot_sig;	/* 指明后三个域可用 */
    uint32_t bs_vol_ID;		/* volume ID */
    uint8_t  bs_vol_lab[11];	/* volume label */
    uint8_t  bs_file_sys_type[8];/* 分区类型 */
};

/* directory struct 32B */
struct dir_entry{
    uint8_t  dir_name[11];	/* 目录名 */
    uint8_t  dir_attr;		/* 属性 */

#define	ATTR_READ_ONLY	0x01	/* 只读 */
#define	ATTR_HIDDEN	0x02	/* 隐藏 */
#define	ATTR_SYSTEM	0x04	/* 系统 */
#define	ATTR_VOLUME_ID	0x08	/* 卷标 */

#define	ATTR_DIRECTORY	0x10	/* 目录 */
#define	ATTR_ARCHIVE	0x20	/* 归档 */

#define	ATTR_LONG_NAME  (ATTR_READ_ONLY|ATTR_HIDDEN|ATTR_SYSTEM|ATTR_VOLUME_ID)	/* 长文件名 */

#define ATTR_LONG_NAME_MASK (ATTR_READ_ONLY|ATTR_HIDDEN|ATTR_SYSTEM|ATTR_VOLUME_ID|ATTR_DIRECTORY|ATTR_ARCHIVE) /* 长文件名掩码 */
    uint8_t  NT_rsvd;			/* for NT use,normal set 0 */
    uint8_t  creative_time_teenth;	/* 文件创建时间毫秒戳(0~199) */
    uint16_t creative_time;		/* 文件创建时间 */
    uint16_t creative_date;		/* 文件创建日期 */
    uint16_t last_access_date;		/* 最后访问日期 */
    uint16_t first_cluster_high;	/* 目录项簇号高字节(FAT16此值为0) */
    uint16_t write_time;		/* 最后写时间 */
    uint16_t write_date;		/* 最后写日期 */
    uint16_t first_cluster_low;		/* 目录项簇号低字节 */
    uint32_t file_size;			/* 文件大小单位是（字节）*/
};

/* 长文件名目录项32B */
struct long_name_entry{	
    /* 长目录项在本组中序号(0x40|N表示该组最后一个长目录顶) */
    uint8_t  ldir_ord;	
    uint8_t  ldir_name1[10];		/* 长文件名1-5个字符(UNICODE) */
    uint8_t  ldir_attr;			/* 必须为0x0f */
    uint8_t  ldir_type;			/* 0 */
    uint8_t  ldir_chksum;		/* 短文件名检验和 */
    uint8_t  ldir_name2[12];		/* 长文件名6-11个字符(UNICODE) */
    uint16_t ldir_first_cluster_low;	/* 0无意义 */
    uint8_t  ldir_name3[4];		/* 长文件名12-13字符(UNICODE) */
};

/* 本项目用到的目录项 */
struct mp3_entry{
    uint8_t	file_name[11];	/* 文件名 */
    uint32_t 	first_cluster;	/* 文件起始簇号 */
    uint32_t 	file_size;	/* 文件大小(字节) file_size/(1024UL*1024UL) = size MB */
};


/* function prototype */
void fat_init(void);
void fat_read_sector(uint32_t sector, uint8_t *sector_buffer);
uint32_t fat_first_sector_of_cluster(uint32_t cluster);
uint32_t fat_next_cluster(uint32_t current_cluster);
void fat_search(uint8_t *entry_number, struct mp3_entry *file_entry);

uint16_t fat_get_bytes_per_sector(void);
uint8_t  fat_get_sectors_per_cluster(void);
uint32_t fat_get_first_fat_sector(void);
uint32_t fat_get_first_data_sector(void);
uint16_t fat_get_first_root_sector(void);
uint16_t fat_get_root_dir_sectors(void);
uint32_t fat_get_root_cluster(void);
uint16_t fat_get_capacity(void);

#endif
