
/* ******************************************************
 * project	: mowin m64+vs1003+sd+lcd3310 mp3 player
 *		:
 * platform	: avr-gcc-4.1.2
 * 		: avr-binutils-2.17
 * 		: avr-libc-1.4.8
 * 		: editor : VIM at Fedora8
 *		:
 * author	: mowin
 * date		: 2008-07-02
 * *****************************************************/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>

#include "spi/spi.h"
#include "lcd/lcd.h"
#include "vs1003/vs1003b.h"
#include "sd/sd.h"
#include "fat/fat.h"
#include "key.h"

#include "usart/usart.h"

extern uint8_t FAT32_enable;
extern PROGMEM uint16_t volume[];

uint8_t total_songs;
struct mp3_entry file_entry;

EEMEM uint8_t play_no_eeprom;
EEMEM uint8_t volume_index_eeprom;
uint8_t volume_index;
uint8_t play_no;
uint32_t cluster_no;
uint32_t sector_no;
uint8_t sector_offset;
uint16_t write_cnt;

uint8_t key_delay_cnt;

void device_init(void);
void set_volume(void);
uint8_t scan_key(void);

/* **********************************
 * function	: main
 * argument	: void
 * return	: 0
 * *********************************/
int main(void)
{
    uint8_t buffer[512];
    uint8_t i;

    device_init();

    wdt_enable(WDTO_1S);/* enable watch dog 1S overflow */
    wdt_reset();

    total_songs = 0;
    fat_search(&total_songs, &file_entry);

    put_string_f(PSTR("\nTotal files: "));
    put_int(total_songs);
    put_cr_lf();

    lcd_screen_clean();
    if(total_songs == 0){
	lcd_put_string(PSTR("No music file!"));
	lcd_set_location(0, 1);
	lcd_put_string(PSTR("Check SD card"));
	lcd_set_location(0, 2);
	lcd_put_string(PSTR("and reset."));
	do{
	    wdt_reset();
	}while(1);
    }

    lcd_put_int(fat_get_capacity());
    if(FAT32_enable == FALSE){
	lcd_put_string(PSTR("MB FAT16"));
    } else {
	lcd_put_string(PSTR("MB FAT32"));
    }

    play_no = eeprom_read_byte(&play_no_eeprom);
    if(play_no < 1 || play_no > total_songs)
	play_no = 1;

    volume_index = eeprom_read_byte(&volume_index_eeprom);
    if(volume_index > 23)
	volume_index = 13;/* 默认音量 */

    set_volume();

starting_play:
    vs1003b_soft_reset();
    while(CHK_DREQ == 0){
	wdt_reset();
    }

    eeprom_write_byte(&play_no_eeprom, play_no);/* 保存播放位置 */

    put_string_f(PSTR("Now playing "));
    put_int(play_no);
    put_string_f(PSTR(" of "));
    put_int(total_songs);
    put_cr_lf();

    lcd_set_location(0, 1);
    lcd_put_string(PSTR("              "));/* clear line */
    lcd_set_location(0, 1);
    lcd_put_string(PSTR("Playing "));
    lcd_put_int(play_no);
    lcd_put_char('/');
    lcd_put_int(total_songs);

    /* 按播放序号查找文件 */
    fat_search(&play_no, &file_entry);

    /* 文件首簇号 */
    cluster_no = file_entry.first_cluster;
    for(;;){
	/* 簇第一扇区号 */
	sector_no = fat_first_sector_of_cluster(cluster_no);

	for(sector_offset = 0; sector_offset < fat_get_sectors_per_cluster(); sector_offset++){
	    /* 分别读取簇内每一扇区 */
	    fat_read_sector(sector_no + sector_offset, buffer);

	    /* 以每次32B固定长度送给VS1003B */
	    write_cnt = 0;
	    for(;;){
		wdt_reset();

		if(CHK_DREQ != 0){
		    ENABLE_DATA_CS;
		    for(i = 32; i > 0; i--)
			vs_write_data(buffer[write_cnt++]);
		    DISABLE_CS;
		}
		/* 送一扇区数据跳出 */
		if(write_cnt >= 512)
		    break;
	    }
	}

	/* Next cluster */
	cluster_no = fat_next_cluster(cluster_no);

	/* 文件是否结束 */
	if(cluster_no >= 0x0ffffff8 || (FAT32_enable == FALSE && cluster_no >= 0xfff8)){
	    if(++play_no > total_songs)
		play_no = 1;

	    goto starting_play;
	}

	/* 扫描按键 */
	i = scan_key();
	if(i == NO_KEY){
	    /* void */
	}
	else if(i == NEXT_KEY){
	    if(++play_no > total_songs)
		play_no = 1;
	    goto starting_play;
	}
	else if(i == VOL_INC_KEY){
	    if(volume_index < 23){
		/* 音量加 */
		volume_index++;
		set_volume();
	    }
	}
	else if(i == VOL_DEC_KEY){
	    if(volume_index > 0){
		/* 音量减 */
		volume_index--;
		set_volume();
	    }
	}
	else if(i == PREV_KEY){
	    if(--play_no < 1)
		play_no = total_songs;
	    goto starting_play;
	}
	else if(i == PLAY_KEY){
	    //put_string_f(PSTR("\nPLAY"));
	}

	/* 接收串口控制码 */
	if(CHECK_RXC){
	    i = UDR0;
	    if(i == '~'){
		/* bootloader上位机复位码 */
		/* 等待看门狗复位以进入bootloader */
		while(1);
	    }
	    else if(i == 'n'){
		if(++play_no > total_songs)
		    play_no = 1;
		goto starting_play;
	    }
	    else if(i == 'b'){
		if(--play_no < 1)
		    play_no = total_songs;
		goto starting_play;
	    }
	    else if(i == '+'){
		if(volume_index < 23){
		    /* 音量加 */
		    volume_index++;
		    set_volume();
		}
	    }
	    else if(i == '-'){
		if(volume_index > 0){
		    /* 音量减 */
		    volume_index--;
		    set_volume();
		}
	    }
	}
    }

    for(;;){} /* 异常，等待看门狗复位 */

    return 0;
}

/* **********************************
 * function	: 外设初始化
 * argument	: void
 * return	: void
 * *********************************/
void device_init(void)
{
    /* key */
    KEY_PORT |= KEY_MASK;

    /* uart */
    usart_init();
    put_string_f(PSTR("usart ok\n"));

    /* spi */
    spi_io_init();

    /* 设置SPI为低速 */
    enable_spi_master(LOW_SPEED);

    /* vs1003b */
    vs1003_init();

    /* lcd */
    lcd_init();
    lcd_set_location(0, 0);
    lcd_put_string(PSTR("Usart 38400bps"));/* 每行显示14个ascii */
    lcd_set_location(0, 1);
    lcd_put_string(PSTR("Vs1003b Tested"));

    /* sd card */
    lcd_set_location(0, 2);
    lcd_put_string(PSTR("Insert SD Card"));
    while(sd_init()){}

    /* 设置SPI为高速 */
    enable_spi_master(HIGH_SPEED);

    /* fat */
    fat_init();
}

/* **********************************
 * function	: 音量设置显示及保存
 * argument	: void
 * return	: void
 * *********************************/
void set_volume(void)
{
    /* 保存音量至EEPROM */
    eeprom_write_byte(&volume_index_eeprom, volume_index);
    /* 设置音量 */
    while(CHK_DREQ == 0){}
    vs_write_cmd(0x0b, pgm_read_word(&volume[volume_index]));

    /* Send to uart */
    put_string_f(PSTR("Volume  "));
    put_int(volume_index);
    put_cr_lf();

    /* Display on LCD */
    lcd_set_location(0, 2);
    lcd_put_string(PSTR("Volume "));
    lcd_put_int(volume_index);
}

/* **********************************
 * function	: 按键状态机
 * argument	: void
 * return	: 键码
 * *********************************/
uint8_t scan_key(void)
{
    static uint8_t key_status = 0;
    uint8_t kc;
    uint8_t ret = NO_KEY;

    /* read key pin regester */
    kc = KEY_PIN & KEY_MASK;

    /* FSM */
    switch(key_status){
	case 0:
	    if(kc != NO_KEY_VALUE)
		key_status = 1; /* 有按键转入消抖动及判断键码 */
	    break;
	case 1:
	    if(kc == NO_KEY_VALUE)
		key_status = 0; /* 抖动 */
	    else if(kc == NEXT_VALUE){ /* 确认，判断并返回键码 */
		ret = NEXT_KEY;
		key_status = 2;
	    }
	    else if(kc == VOL_INC_VALUE){
		ret = VOL_INC_KEY;
		key_status = 2;
	    }
	    else if(kc == VOL_DEC_VALUE){
		ret = VOL_DEC_KEY;
		key_status = 2;
	    }
	    else if(kc == PREV_VALUE){
		ret = PREV_KEY;
		key_status = 2;
	    }
	    else if(kc == PLAY_VALUE){
		ret = PLAY_KEY;
		key_status = 2;
	    }
	    break;
	case 2:
	    if(kc == NO_KEY_VALUE)
		key_status = 0; /* 按键已经释放 */
	    break;
    }

    return ret;
}
/* end of main.c */
