
/* sd app for mp3 player */

#define SD_DEBUG 1

#include <avr/io.h>

#include "../spi/spi.h"
#include "sd.h"

#if SD_DEBUG
#include <avr/pgmspace.h>
#include "../usart/usart.h"
#endif

uint8_t (*sd_access_byte)(uint8_t d) = spi_access_byte;

/* ***********************************
 * function	: 发送命令
 * argument	: cmd 命令
 * 		: a   地址
 * 		:
 * return 	: SD返回码(available except 0xff)
 * **********************************/
uint8_t sd_send_cmd(uint8_t cmd, uint32_t a)
{
    uint8_t i, cnt;

    //(*sd_access_byte)(0xff);//发送8 个时钟
    sd_access_byte(0xff);//发送8 个时钟
    sd_access_byte(cmd | 0x40);//送头命令
    sd_access_byte(a >> 24);
    sd_access_byte(a >> 16);
    sd_access_byte(a >> 8);
    sd_access_byte(a);
    sd_access_byte(0x95);//仅对RESET有效的CRC效验码
    //等待回应
    cnt = 0xff;
    do{
	i = sd_access_byte(0xff);
	if(--cnt == 0){
	    break;
	}
    }while(i == 0xff);

    return i;
}



/* ***********************************
 * function	: SD初始化设置
 * 		: 要先设置VS1003B端口否则SD初始化失败
 * argument	: void
 * return	: 0 success, 1 fail
 * **********************************/
uint8_t sd_init(void)
{
    uint8_t i, cnt;

    SD_PORT |= _BV(SD_CS);
    SD_DDR |= _BV(SD_CS);

    cnt = 0xff;
    while(--cnt);

    ENABLE_SD;
    cnt = 0xff;
    do{
	for(i = 10; i > 0; i--)
	    sd_access_byte(0xff);//发送74 个时钟

	i = sd_send_cmd(0, 0); //发送IDLE 命令
	if(--cnt == 0){
	    DISABLE_SD;

#if SD_DEBUG
	    put_string_f(PSTR("Response: "));
	    put_int(i);
	    put_string_f(PSTR(" IDLE command timeout\n"));
#endif

	    return 1;//time out
	}
    }while(i != 1);//返回1执行成功

    cnt = 0xff;
    do{
	i = sd_send_cmd(1, 0);//发送active 命令
	if(--cnt == 0){
	    DISABLE_SD;

#if SD_DEBUG
	    put_string_f(PSTR("Response: "));
	    put_int(i);
	    put_string_f(PSTR(" ACTIVE command timeout\n"));
#endif

	    return 1;//time out
	}
    }while(i != 0);//返回0执行成功

    sd_send_cmd(59, 0);//disable CRC
    sd_send_cmd(16, 512);//设置扇区为512B

    DISABLE_SD;
    return 0;//正常返回
}

/* ***********************************
 * function	: reading a block form SD card
 * 		:
 * arguments	: sector number(address = sector * 512)
 * 		: reading buffer array (512B)
 *		:
 * return	: 0 success, 1 fail
 * **********************************/
uint8_t sd_read_block(uint32_t sector, uint8_t *sd_buffer)
{
    uint8_t i;
    uint16_t cnt;

    ENABLE_SD;
    i = sd_send_cmd(17, sector << 9);//17读命令, addr = sector << 9 == sector * 512
    if(i != 0){
	DISABLE_SD;
	return i;
    }

    //等待数据开始
    cnt = 0xffff;
    do{
	i = sd_access_byte(0xff);
	if(--cnt == 0){
	    DISABLE_SD;
	    return 1;
	}
    }while(i != 0xfe);//0xfe 表示数据开始

    for(cnt = 0; cnt < 512; cnt++)
	*sd_buffer++ = sd_access_byte(0xff);

    sd_access_byte(0xff);
    sd_access_byte(0xff);//CRC

    DISABLE_SD;
    return 0;	
}

//end of file
