
//spi.c

#include <avr/io.h>
#include "spi.h"

/* *******************************
 * function	: IO 初始化
 * argument	: void
 * return	: void
 * ******************************/
void spi_io_init(void)
{
    //MISO 设为输入
    //MOSI，SCK 设为输出，SS 也设为输出
    //SS（PB0）脚为SD 卡CS 脚
    SPI_PORT |= (_BV(SPI_MOSI)|_BV(SPI_SCK)|_BV(SPI_SS)|_BV(SPI_MISO));
    SPI_DDR |= (_BV(SPI_MOSI)|_BV(SPI_SCK)|_BV(SPI_SS));
    SPI_DDR &=~ _BV(SPI_MISO);
}

/* *******************************
 * function	: 设置SPI 主模式工作速度
 * argument	: void
 * return	: void
 * ******************************/
void enable_spi_master(uint8_t spi_speed)
{
    if(spi_speed == HIGH_SPEED){
	//使能 SPI主机模式,设置时钟速率为 fck/2
	SPCR = _BV(SPE)|_BV(MSTR);
	SPSR |= _BV(SPI2X);
    } 
    else if(spi_speed == LOW_SPEED){
	//使能 SPI主机模式,设置时钟速率为 fck/128
	SPCR = _BV(SPE)|_BV(MSTR)|_BV(SPR1)|_BV(SPR0);
	SPSR &=~ _BV(SPI2X) ;
    }
}

/* *******************************
 * function	: 读一个字节i = spi_access_byte(0xff);
 *		: 写一个字节spi_access_byte(data);
 * argument	: d 写入的数据
 * return	: 读出的数据
 * ******************************/
uint8_t spi_access_byte(uint8_t d)
{
    SPDR = d;
    do ;
    while(!CHKBIT(SPSR, SPIF)); 
    return SPDR;
}

//end of file
