

/* *********************************
 * usart hardware function
 * for atmega64
 * ********************************/

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "usart.h"

/* *********************************
 * function	: 设置串口参数
 * argument	: void
 * return	: void
 * ********************************/
void usart_init(void) 
{
    /*设置波特率*/
    UBRR0H = UBRRH_VALUE;
    UBRR0L = UBRRL_VALUE; 
#if U2X_
    UCSR0A |= _BV(U2X);
#endif
    /*接收器与发送器使能*/
    UCSR0B = _BV(RXEN)|_BV(TXEN);
    /*设置帧格式: 8 个数据位, 1个停止位*/
    UCSR0C = _BV(UCSZ1)|_BV(UCSZ0);
}

/* *********************************
 * function	: 发送单个字节至串口
 * argument	: char
 * return	: void
 * ********************************/
void write_udr(unsigned char c)
{
    UDR0 = c;
    while(!CHKBIT(UCSR0A, TXC));
#if U2X_
    UCSR0A = _BV(TXC)|_BV(U2X);
#else
    UCSR0A = _BV(TXC);
#endif
}

/* *********************************
 * function	: 接收单个字节
 * argument	: void
 * return	: char
 * ********************************/
unsigned char read_udr(void)
{
    while(!CHKBIT(UCSR0A, RXC));
    return UDR0;
}

/* *********************************
 * function	: 发送ram字符串
 * argument	: 字符串指针
 * return	: void
 * ********************************/
void put_string_ram(const char *str)
{
    char c;
    do{
	c = *str++;

	if(c == '\n'){
	    write_udr(0x0d);
	    write_udr(0x0a);
	}
	else if(c == '\0')
	    break;
	else
	    write_udr(c);
    } while(1);
}

/* *********************************
 * function	: 发送flash字符串
 * argument	: 字符串指针
 * return	: void
 * ********************************/
void put_string_f(const char *str)
{
    char c;
    do{
	c = pgm_read_byte(str++);

	if(c == '\n'){
	    write_udr('\r');
	    write_udr('\n');
	} 
	else if(c == '\0')
	    break;
	else 
	    write_udr(c);
    } while(1);
}

/* **************************
 * function	: usart put data
 * 		: 显示整数
 * argument	: -9999 ~ 9999
 * return 	: void
 * *************************/
void put_int(int16_t d)
{
    int16_t temp, middle;

#define FALSE 0
#define TRUE  1
    uint8_t disp_en = FALSE;

    if(d & _BV(15)){
	/* 负数 */
	temp = d - 1;
	temp = ~temp;
	write_udr('-');
    } else {
	/* 正数 */
	temp = d;
    }

    middle = temp / 1000;
    if(middle){
	disp_en = TRUE;
	write_udr(middle + 0x30);/* 千位 */
    }

    temp = temp - (middle * 1000);
    middle = temp / 100;
    if(middle || disp_en == TRUE){
	disp_en = TRUE;
	write_udr(middle + 0x30);/* 百位 */
    }

    temp = temp - (middle * 100);
    middle = temp / 10;
    if(middle || disp_en == TRUE){
	write_udr(middle + 0x30);/* 十位 */
    }

    temp = temp -(middle * 10);
    middle = temp;
    write_udr(middle + 0x30);/* 个位 */
}

void put_cr_lf(void)
{
    write_udr('\r');
    write_udr('\n');
}
/********** end of file ***********/
